<?php
require_once OWPPOCZTA_PATH.'models/14/Context.php';
require_once OWPPOCZTA_PATH.'models/14/OwppocztaLink.php';
require_once OWPPOCZTA_PATH.'controllers/OwppocztaCommon.php';

class OwppocztaBase extends OwppocztaCommon
{
    protected $context;

    public function __construct()
    {
        $this->context = new Context();
        return parent::__construct();
    }

    /**
     * Funkcja instalująca w wersji 1.4 zmienia selektor typu przesyłki
     * @return bool
     */
    public function install()
    {
        if (!parent::install() ||
            !Configuration::updateValue('MOD_OWPPOCZTA_CARRIER_LIST','#carrierTable .radio input[type="radio"]'))
            return false;
        return true;
    }

    protected function registerHooks()
    {
        if( !$this->registerHook('updateCarrier') ||
            !$this->registerHook('extraCarrier') ||
            !$this->registerHook('processCarrier') ||
            !$this->registerHook('adminOrder') ||
            !$this->registerHook('backOfficeTop') ||
            !$this->registerHook('orderDetail') ||
            !$this->registerHook('orderConfirmation'))
            return false;
        return true;
    }

    protected function unregisterHooks()
    {
        if( !$this->unregisterHook('updateCarrier') ||
            !$this->unregisterHook('extraCarrier') ||
            !$this->unregisterHook('processCarrier') ||
            !$this->unregisterHook('adminOrder') ||
            !$this->unregisterHook('backOfficeTop') ||
            !$this->unregisterHook('orderDetail') ||
            !$this->unregisterHook('orderConfirmation'))
            return false;
        return true;
    }

    public function showMap()
    {
        $this->display_header = false;
        $this->display_footer = false;
        $post_office = OwppocztaPostOffice::get_post_by_pni(Tools::getValue("pni"));
        $this->context->smarty->assign('address_array', $post_office);
        return $this->context->smarty->display(OWPPOCZTA_PATH.'views/templates/front/show_post_map.tpl');
    }

    public function hookadminOrder($params)
    {
        $order = new Order($params['id_order']);
        if(in_array((int)$order->id_carrier, $this->connected_carriers)) {
            $carrier = new Carrier((int)$order->id_carrier);

            $owppoczta_order = new OwppocztaOrder((int)$order->id_cart);
            $address_array = OwppocztaPostOffice::get_post_by_pni($owppoczta_order->getPni());

            $this->context->smarty->assign('carrier_name', $carrier->name);
            $this->context->smarty->assign('poczta_adr', $address_array);

            if ($carrier->external_module_name == 'owppoczta') {
                echo $this->display(OWPPOCZTA_PATH, 'views/templates/hook/backorder' . OWPPOCZTA_PS_VERSION . '.tpl');
            }
        }
    }

    public function hookbackOfficeTop($params)
    {
        echo parent::hookdisplayBackOfficeTop($params);
    }

    public function hookOrderDetail($params)
    {
        if(in_array($params['order']->id_carrier, $this->connected_carriers)) {

            $owppoczta_order = new OwppocztaOrder((int)$params['order']->id_cart);
            $post_office = OwppocztaPostOffice::get_post_by_pni($owppoczta_order->getPni());
            if (is_array($post_office)) {
                $link = $this->context->link->getModuleLink('owppoczta', 'map', array(), Configuration::get('PS_SSL_ENABLED') ? true : false);

                $sign = (strpos($link, '?') === false ? '?' : '&');

                $this->context->smarty->assign(array(
                    'poczta' => $post_office,
                    'map_url' => $link,
                    'sign' => $sign
                ));
                echo $this->display(OWPPOCZTA_PATH, 'views/templates/hook/order_detail.tpl');
            }
        }
    }

    public function hookorderConfirmation($params)
    {
        if(in_array($params['objOrder']->id_carrier, $this->connected_carriers))
        {
            $owppoczta_order = new OwppocztaOrder((int)$params['objOrder']->id_cart);
            $post_office = OwppocztaPostOffice::get_post_by_pni($owppoczta_order->getPni());
            if (is_array($post_office))
            {
                $link = $this->context->link->getModuleLink('owppoczta', 'map', array(), Configuration::get('PS_SSL_ENABLED') ? true : false);

                $sign = (strpos($link, '?') === false ? '?' : '&');

                $this->context->smarty->assign(array(
                    'poczta'    => $post_office,
                    'map_url'   => $link,
                    'sign'      => $sign
                ));
                echo $this->display(OWPPOCZTA_PATH, 'views/templates/hook/order_detail.tpl');
            }
        }
    }

    public function hookextraCarrier($params)
    {
        $owppoczta_order = new OwppocztaOrder((int)$params['cart']->id);
        $post_office = OwppocztaPostOffice::get_post_by_pni($owppoczta_order->getPni());

        $this->hookExtraCarrierHelper($post_office);

        return $this->display(OWPPOCZTA_PATH, 'views/templates/hook/extra_carrier.tpl');
    }

    public function ajax_updateDatabaseAction()
    {
        global $cookie;
        $is_admin = ($cookie->profile == _PS_ADMIN_PROFILE_);
        $token = Tools::getValue('token');
        if ($is_admin || (!is_null($token) && $token != false &&
                $token == Configuration::get('MOD_OWPPOCZTA_TOKEN') &&
                Configuration::get('MOD_OWPPOCZTA_DATABASE_STATUS') > 0))
        {
            return $this->updateDatabase();
        }
    }

    /**
     * Wyświetla menu konfiguracji modułu
     * Dodatkowo sprawdza, czy nie trzeba zaktualizować bazy danych
     * (w Prestashop 1.5+ wbudowane)
     *
     * @return mixed
     */
    public function getContent()
    {
        if($this->checkIfOldVersionDatabaseExists())
        {
            require_once(OWPPOCZTA_PATH . 'upgrade/install-1.0.php');
            upgrade_module_1_0($this);
        }
        return parent::getContent();
    }

    /**
     * Sprawdza, czy baza danych została zaktualizowana do wersji 1.0+ modułu
     *
     * @return bool
     */
    protected function checkIfOldVersionDatabaseExists()
    {
        return ( count(Db::getInstance()->executeS('SHOW COLUMNS FROM `' . _DB_PREFIX_ . 'cart` LIKE "owppoczta_ext"') ) > 0 ) ? TRUE : FALSE;
    }
}

