<?php
require_once OWPPOCZTA_PATH . 'models/OwppocztaPostOffice.php';
require_once OWPPOCZTA_PATH . 'models/OwppocztaCarrier.php';
require_once OWPPOCZTA_PATH . 'models/OwppocztaOrder.php';
require_once OWPPOCZTA_PATH . 'models/OwppocztaConnection.php';
require_once OWPPOCZTA_PATH . 'models/OwppocztaUpdater.php';
require_once OWPPOCZTA_PATH . 'models/OwppocztaLogger.php';
require_once OWPPOCZTA_PATH . 'models/JsonEncoder.php';

/**
 * Class OwppocztaCommon
 */
class OwppocztaCommon extends Module
{
    public $id_carrier;

    protected $_html = '';

    protected $_postErrors = array();

    protected $_moduleName = 'owppoczta';

    protected $connected_carriers;

    /**
     * @return array
     */
    public function getConnectedCarriers()
    {
        if (is_null($this->connected_carriers))
            $this->connected_carriers = OwppocztaConnection::getConnections();
        return $this->connected_carriers;
    }

    /**
     * @param array $connected_carriers
     */
    public function setConnectedCarriers($connected_carriers)
    {
        $this->connected_carriers = $connected_carriers;
    }

    //obsługiwane typy miejsc odbioru przesyłki
    /**
     * @var
     */
    protected $placeTypesList;

    public function getPlaceTypesList()
    {
        if (is_null($this->placeTypesList))
            $this->placeTypesList = unserialize(Configuration::get('MOD_OWPPOCZTA_PLACE_TYPES'));
        return $this->placeTypesList;
    }

    public $extra_mail_vars;

    /**
     * Konstruktor modułu
     */
    public function __construct()
    {
        $this->name = 'owppoczta';
        $this->dname = 'owppoczta';
        $this->tab = 'shipping_logistics';
        $this->version = '1.3.4';
        $this->author = 'MM Studio - michalmleczko.waw.pl';
        $this->need_instance = 0;

        parent::__construct();

        $this->displayName = $this->l('Odbiór w punkcie - Poczta Polska');
        $this->description = $this->l('Moduł pozwala na korzystanie w sklepie z jednej z wygodniejszych form odbioru przesyłek – paczka dostarczona zostanie do wskazanej placówki pocztowej, a jej adresat powiadomiony SMS-em albo e-mailem o możliwości odbioru. Moduł umożliwia klientom sklepu wybranie najodpowiedniejszej placówki z ponad 4200 dogodnie zlokalizowanych punktów Poczty Polskiej.');

        $this->confirmUninstall = $this->l('Czy jesteś pewien, że chcesz odinstalować ten moduł? Pamiętaj, że nie zostaną usunięte utworzone w trakcie instalacji nowe typy przesyłek.');

        $this->context->smarty->assign(array(
            'module_name' => $this->dname,
            'rmodule_name' => $this->name));

        if (self::isInstalled($this->name)) {
            // Getting carrier list
            global $cookie;
            $carriers = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL, PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);

            // Saving id carrier list
            $id_carrier_list = array();
            foreach ($carriers as $carrier)
                $id_carrier_list[] .= $carrier['id_carrier'];
        }
    }

    /**
     * Funkcja instalująca moduł
     * @return bool
     */
    public function install()
    {
        if (
            !$this->createTables() ||
            !parent::install() ||
            !$this->_createContent() ||
            !$this->registerHooks() ||
            !$this->set_token()
        )
            return false;
        return true;
    }

    /**
     * Tworzenie tabel
     * @return bool
     */
    protected function createTables()
    {
        // stara metoda na przechowywanie numeru PNI poczty w tabeli,cart, bez oddzielnej tabeli owppoczta_orders
        //$sql1 = 'ALTER TABLE `' . _DB_PREFIX_ . 'cart`  ADD `owppoczta_ext` TEXT NOT NULL';

        /*TWORZENIE TABEL*/
        //dodanie tabeli na numer PNI poczty w danym zamówieniu
        $sql1 = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'owppoczta_orders` (
	    `id_cart` INT(10) NOT NULL,
	    `pni` INT(6) NOT NULL,
	    `id_carrier` INT(10) NOT NULL,
	    PRIMARY KEY (`id_cart`))';

        //stworzenie tabeli z 5 najbliższymi pocztami dla danego kodu
        $sql2 = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'placowki_poczty_pnapnipp` (
        `PNA` VARCHAR( 6 ) NOT NULL ,
        `PNI` INT( 6 ) NOT NULL,
        PRIMARY KEY ( `PNI` , `PNA` )
        )';

        //stworzenie tabeli z danymi placówek pocztowych
        $sql3 = "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . "placowki_poczty` (
        `PNI` MEDIUMINT( 6 ) NOT NULL ,
        `y` DECIMAL( 9, 6 ) NOT NULL ,
        `x` DECIMAL( 9, 6 ) NOT NULL ,
        `wojewodztwo` ENUM( 'DOLNOŚLĄSKIE', 'KUJAWSKO-POMORSKIE', 'LUBELSKIE', 'LUBUSKIE', 'ŁÓDZKIE', 'MAŁOPOLSKIE', 'MAZOWIECKIE', 'OPOLSKIE', 'PODKARPACKIE', 'PODLASKIE', 'POMORSKIE', 'ŚLĄSKIE', 'ŚWIĘTOKRZYSKIE', 'WARMIŃSKO-MAZURSKIE', 'WIELKOPOLSKIE', 'ZACHODNIOPOMORSKIE' ) NOT NULL ,
        `powiat` VARCHAR( 50 ) NOT NULL ,
        `gmina` VARCHAR( 50 ) NOT NULL ,
        `nazwa_placowki` VARCHAR( 100 ) NOT NULL ,
        `kod_pocztowy` CHAR( 6 ) NOT NULL ,
        `miejscowosc` VARCHAR( 100 ) NOT NULL ,
        `ulica` VARCHAR( 100 ) NOT NULL ,
        `telefon` VARCHAR( 150 ) NOT NULL ,
        `godziny_pracy` TEXT NOT NULL ,
        `stan_placowki` VARCHAR( 50 ) NOT NULL ,
        `typ` VARCHAR( 10 ) NOT NULL ,
        PRIMARY KEY ( `PNI` )
        )";

        $sql4 = sprintf('CREATE TABLE IF NOT EXISTS %sowppoczta_connections (
	    id INT(10) NOT NULL AUTO_INCREMENT,
	    id_carrier INT(10) NOT NULL,
	    owppocztacarrier_name VARCHAR(10) NOT NULL,
	    INDEX id (id))', _DB_PREFIX_);

        if (!Db::getInstance()->execute($sql1) ||
            !Db::getInstance()->execute($sql2) ||
            !Db::getInstance()->execute($sql3) ||
            !Db::getInstance()->execute($sql4)
        )
            return false;
        return true;
    }

    /**
     * Rejestrowanie hooków
     *
     * @return bool
     */
    protected function registerHooks()
    {
        if (!$this->registerHook('updateCarrier') ||
            !$this->registerHook('extraCarrier') ||
            !$this->registerHook('processCarrier') ||
            !$this->registerHook('displayAdminOrder') ||
            !$this->registerHook('displayBackOfficeTop') ||
            !$this->registerHook('displayOrderDetail') ||
            !$this->registerHook('displayOrderConfirmation') ||
            !$this->registerHook('displayHeader')
        )
            return false;
        return true;
    }

    /**
     * @return bool
     */
    public function uninstall()
    {
        // Uninstall

        if (!parent::uninstall() ||
            !$this->dropTables() ||
            !$this->_deleteContent() ||
            !$this->unregisterHooks()
        )
            return false;
        return true;
    }

    /**
     * @return bool
     */
    protected function unregisterHooks()
    {
        if (
            !$this->unregisterHook('updateCarrier') ||
            !$this->unregisterHook('extraCarrier') ||
            !$this->unregisterHook('processCarrier') ||
            !$this->unregisterHook('displayAdminOrder') ||
            !$this->unregisterHook('displayBackOfficeTop') ||
            !$this->unregisterHook('displayOrderDetail') ||
            !$this->unregisterHook('displayOrderConfirmation') ||
            !$this->unregisterHook('header')
        )
            return false;
        return true;
    }

    /**
     * Funkcja usuwająca tablice przy odinstalowywaniu
     * poprzednia wersja usuwała dodatkową kolumnę w tabeli 'cart'
     * teraz robi to skrypt aktualizujący do wersji 1.0
     *
     * @return bool
     */
    protected function dropTables()
    {
        $sql = sprintf('DROP TABLE %1$sowppoczta_orders, %1$splacowki_poczty, %1$splacowki_poczty_pnapnipp, %1$sowppoczta_connections;',
            _DB_PREFIX_);

        return Db::getInstance()->execute($sql);
    }


    /**
     * Funkcja ustalająca token (jednorazowo, przy instalacji), służący potem autoryzacji przy
     * aktualizacji z linka za pomocą CRONa
     * @return bool
     */
    protected function set_token()
    {
        return Configuration::updateValue('MOD_OWPPOCZTA_TOKEN', md5(time() . __PS_BASE_URI__));
    }

    /**
     * Dodaje połączenie sposobu wysyłki z modułem
     *
     * @param $carrier_id
     * @param $place_type_name - typ placówki, którego dotyczy połączenie
     * @return bool
     */
    public function addConnectedCarrier($carrier_id, $place_type_name)
    {
        $connected_carrier = new OwppocztaConnection($carrier_id, $place_type_name);
        return $connected_carrier->save();
    }

    /**
     * Usuwa połączenie sposobu wysyłki z modułem
     *
     * @param $carrier_id
     * @param $place_type_name - od wersji 1.3
     * @return bool
     */
    public function removeConnectedCarrier($carrier_id, $place_type_name = null)
    {
        $connected_carrier = new OwppocztaConnection($carrier_id, $place_type_name);
        return $connected_carrier->delete();
    }

    /**
     * Sprawdza, czy nie mamy nieprawidłowo połączonych sposobów przesyłki
     *
     * @return void
     */
    protected function checkConnectedCarriers()
    {
//        foreach ($this->connected_carriers as $carrier_id) {
//            $external_module = Db::getInstance()->getValue(sprintf('SELECT external_module_name FROM %scarrier WHERE id_carrier = %d', _DB_PREFIX_, $carrier_id));
//            if ($external_module !== 'owppoczta') {
//                $this->removeConnectedCarrier($carrier_id);
//            }
//        }
    }

    /**
     * Funkcja wyświetlająca menu ustawień w backendzie
     *
     * @return mixed
     */
    public function getContent()
    {
        //najpierw sprawdzamy, czy wszystkie aktualnie podłączone sposoby przesyłki są podłączone prawidłowo
        $this->checkConnectedCarriers();

        $this->_displayContent();

        return $this->display(OWPPOCZTA_PATH, 'views/templates/admin/settings' . OWPPOCZTA_PS_VERSION . '.tpl');
    }

    /**
     * @return mixed
     */
    protected function getLang()
    {
        return $this->context->language->id;
    }

    /**
     * Funkcja zwracająca tablicę metod wysyłki, do użycia w panelu konfiguracji
     * Wyróżnia metody wysyłki połączone z modułem
     *
     * @return mixed
     */
    protected function getCarriers()
    {
        $carriers = Carrier::getCarriers($this->getLang(), true, false, false, null, '%');
        $connectedCarriers = $this->getConnectedCarriers();
        $carrier['connected'] = null;
        //przyporządkowanie obrazka do każdego sposobu wysyłki
        foreach ($carriers as &$carrier) {
            foreach ($this->getPlaceTypesList() as $place_type) {
                if (
                    $carrier['external_module_name'] === 'owppoczta' &&
                    $this->isCarrierConnected($carrier['id_carrier']) &&
                    in_array($place_type->getName(), $connectedCarriers[$carrier['id_carrier']])
                )
                    $carrier['connected'][] = $place_type->getName();
            }

            $carrier['img'] = file_exists(_PS_SHIP_IMG_DIR_ . (int)$carrier['id_carrier'] . '.jpg') ? _THEME_SHIP_DIR_ . (int)$carrier['id_carrier'] . '.jpg' : false;
        }
        return $carriers;
    }

    /**
     * Pomocnicza funkcja dodająca zmienne smarty do menu konfiguracji
     *
     * @return void
     */
    protected function _displayContent()
    {
        $this->context->smarty->assign(array(
            'carriers' => $this->getCarriers(),
            'place_types' => $this->getPlaceTypesList(),//typy punktów odbioru
            'update_status' => Configuration::get('MOD_OWPPOCZTA_DATABASE_STATUS'),
            'update_interval' => Configuration::get('MOD_OWPPOCZTA_DATABASE_INTERVAL'),
            'logs' => OwppocztaLogger::getLogs(),
            'ajax_url' => $this->context->link->getModuleLink('owppoczta', 'ajax', array(), Configuration::get('PS_SSL_ENABLED') ? true : false),
            'admin_carrier_url' => $this->context->link->getAdminLink('AdminCarrierWizard'),
            'link' => $this->context->link->getModuleLink('owppoczta', 'ajax', array(
                'action' => 'updateDatabase',
                'token' => Configuration::get('MOD_OWPPOCZTA_TOKEN')),
                Configuration::get('PS_SSL_ENABLED') ? true : false),
            'remove_logs' => $this->context->link->getModuleLink('owppoczta', 'ajax', array(
                'action' => 'removeLogs',
                'token' => Configuration::get('MOD_OWPPOCZTA_TOKEN')
            ), Configuration::get('PS_SSL_ENABLED') ? true : false),
            'pp_pnapni_enc' => Configuration::get('MOD_OWPPOCZTA_PP_PNAPNI_ENC'),
            'pp_pnapni_uri' => Configuration::get('MOD_OWPPOCZTA_URI_PNAPNIPP'),
            'pp_enc' => Configuration::get('MOD_OWPPOCZTA_PP_ENC'),
            'pp_uri' => Configuration::get('MOD_OWPPOCZTA_URI_PP'),
            'google_maps_api_key' => Configuration::get('MOD_OWPPOCZTA_GMAPS_API_KEY')
        ));
    }

    /**
     * Standardowa konfiguracja
     * @return bool
     */
    protected function _createContent()
    {
        if (//standardowe opcje
            !Configuration::updateValue('MOD_OWPPOCZTA_DATABASE_STATUS', 1) ||
            !Configuration::updateValue('MOD_OWPPOCZTA_DATABASE_INTERVAL', 86400) ||
            !Configuration::updateValue('MOD_OWPPOCZTA_LAST_UPDATE', 0) ||
            // zaawansowane opcje
            !Configuration::updateValue('MOD_OWPPOCZTA_PP_ENC', 'cp1250') ||
            !Configuration::updateValue('MOD_OWPPOCZTA_PP_PNAPNI_ENC', 'UTF-8') ||
            !Configuration::updateValue('MOD_OWPPOCZTA_GMAPS_API_KEY', 'AIzaSyAhBmnF90rKXc7PtERC3KTZgf3ki21aawU') ||
            //lista typów placówek pocztowych
            !Configuration::updateValue('MOD_OWPPOCZTA_PLACE_TYPES', serialize($this->setPlaceTypesList())) ||
            !Configuration::updateValue('MOD_OWPPOCZTA_URI_PNAPNIPP', 'http://odbiorwpunkcie.poczta-polska.pl/pliki.php?t=pnapni-csv-K48S') ||
            !Configuration::updateValue('MOD_OWPPOCZTA_URI_PP', 'http://odbiorwpunkcie.poczta-polska.pl/pliki.php?t=csvK48S')
        )
            return false;
        return true;
    }

    /**
     * Generuje z predefiniowanych danych listę obsługiwanych typów przesyłek
     *
     * @return array
     */
    public function setPlaceTypesList()
    {
        $orlen = new OwppocztaCarrier('orlen', 'Odbiór na stacjach Orlen');
        $poczta = new OwppocztaCarrier('poczta', 'Odbiór na poczcie');
        return array($poczta, $orlen);
    }

    /**
     * Usuwa konfigurację
     *
     * @return bool
     */
    protected function _deleteContent()
    {
        if (//standardowe opcje
            !Configuration::deleteByName('MOD_OWPPOCZTA_DATABASE_STATUS') ||
            !Configuration::deleteByName('MOD_OWPPOCZTA_DATABASE_INTERVAL') ||
            !Configuration::deleteByName('MOD_OWPPOCZTA_LAST_UPDATE') ||
            !Configuration::deleteByName('MOD_OWPPOCZTA_TOKEN') ||
            //zaawansowane opcje
            !Configuration::deleteByName('MOD_OWPPOCZTA_PP_ENC') ||
            !Configuration::deleteByName('MOD_OWPPOCZTA_PP_PNAPNI_ENC') ||
            !Configuration::deleteByName('MOD_OWPPOCZTA_GMAPS_API_KEY') ||
            !Configuration::deleteByName('MOD_OWPPOCZTA_PLACE_TYPES') ||
            !Configuration::deleteByName('MOD_OWPPOCZTA_URI_PNAPNIPP') ||
            !Configuration::deleteByName('MOD_OWPPOCZTA_URI_PP')
        )
            return false;
        return true;

    }

    /**
     * Dodaje do nagłówka wymagane pliki select2 - style i skrypty
     * @param $params
     */
    public function hookDisplayHeader($params)
    {
        $allowed_controllers = array('order-opc', 'order', 'cart');
        $_controller = $this->context->controller;
        if (isset($_controller->php_self) && in_array($_controller->php_self, $allowed_controllers)) {
            $this->context->controller->addCss(($this->_path) . 'css/owppoczta.css', 'all');
            $this->context->controller->addCss(($this->_path) . 'css/select2.min.css', 'all');
            $this->context->controller->addJs(($this->_path) . 'js/select2.js');
            $this->context->controller->addJs(($this->_path) . 'js/select2pl.js');
            $this->context->controller->addJs(($this->_path) . 'js/owppoczta_carrier_list.js');
            $this->context->controller->addJs('https://maps.googleapis.com/maps/api/js?key=' . Configuration::get('MOD_OWPPOCZTA_GMAPS_API_KEY'));
        }
        $allowed_controllers = array('order-opc', 'order-confirmation', 'order-detail');
        if (isset($_controller->php_self) && in_array($_controller->php_self, $allowed_controllers)) {
            $this->context->controller->addCss(($this->_path) . 'css/owppoczta.css', 'all');
        }
    }

    /**
     * Funkcja wyświetlająca dodatkowe opcje wysyłki przy realizacji zamówienia
     *
     * @param $params
     * @return mixed
     */
    public function hookextraCarrier($params)
    {
        if ($this->isCarrierConnected($params['cart']->id_carrier)) {
            $owppoczta_order = new OwppocztaOrder((int)$this->context->cart->id);
            if ($params['cart']->id_carrier === $owppoczta_order->getIdCarrier()) {
                $post_office = OwppocztaPostOffice::get_post_by_pni($owppoczta_order->getPni());
                $this->hookExtraCarrierHelper($post_office);
            } else {
                $owppoczta_order->remove();
                $this->hookExtraCarrierHelper();
            }
            return $this->display(OWPPOCZTA_PATH, 'extra_carrier.tpl');
        }
        return $this->display(OWPPOCZTA_PATH, 'extra_carrier_not_connected.tpl');;
    }

    /**
     * Funkcja pomocnicza dla hookextraCarrier, ułatwia rozszerzanie w Prestashop 1.4
     *
     * @param $post_office
     * @param $params - standardowe parametry przekazywane w hooku
     */
    public function hookExtraCarrierHelper($post_office = null)
    {
        if (is_array($post_office))
            $this->context->smarty->assign('poczta', $post_office);
        else
            $this->context->smarty->assign('poczta', false);

        $this->context->smarty->assign(array(
                'ajax_url' => $this->context->link->getModuleLink('owppoczta', 'ajax', array(), Configuration::get('PS_SSL_ENABLED') ? true : false)
            )
        );
    }

    /**
     * @param $params
     * @return string
     */
    public function hookdisplayOrderDetail($params)
    {
        if ($this->isCarrierConnected($params['order']->id_carrier)) {
            $owppoczta_order = new OwppocztaOrder((int)$params['order']->id_cart);
            $post_office = OwppocztaPostOffice::get_post_by_pni($owppoczta_order->getPni());
            if (is_array($post_office)) {
                $link = $this->context->link->getModuleLink('owppoczta', 'map', array(), Configuration::get('PS_SSL_ENABLED') ? true : false);
                $sign = (strpos($link, '?') === false ? '?' : '&');
                $this->context->smarty->assign(array(
                    'poczta' => $post_office,
                    'map_url' => $link,
                    'sign' => $sign
                ));
                return $this->display(OWPPOCZTA_PATH, 'order_detail.tpl');
            }
        }
    }

    /**
     * @param $params
     * @return string
     */
    public function hookdisplayOrderConfirmation($params)
    {
        //TODO sprawdzić czy nie powoduje błędów
        //podsumowanie zamówienia
        if ($this->isCarrierConnected((int)$params['objOrder']->id_carrier)) {
            $owppoczta_order = new OwppocztaOrder((int)$params['objOrder']->id_cart);
            $post_office = OwppocztaPostOffice::get_post_by_pni($owppoczta_order->getPni());
            if (is_array($post_office)) {
                $link = $this->context->link->getModuleLink('owppoczta', 'map', array(), Configuration::get('PS_SSL_ENABLED') ? true : false);

                $sign = (strpos($link, '?') === false ? '?' : '&');

                $this->context->smarty->assign(array(
                    'poczta' => $post_office,
                    'map_url' => $link,
                    'sign' => $sign
                ));
                return $this->display(OWPPOCZTA_PATH, 'order_detail.tpl');
            }
        }
    }

    /**
     * @param $params
     * @return string
     */
    public function hookdisplayAdminOrder($params)
    {
        $order = new Order($params['id_order']);
        if ($this->isCarrierConnected((int)$order->id_carrier)) {

            $carrier = new Carrier((int)$order->id_carrier);

            $owppoczta_order = new OwppocztaOrder((int)$order->id_cart);
            $address_array = OwppocztaPostOffice::get_post_by_pni($owppoczta_order->getPni());

            $this->context->smarty->assign('carrier_name', $carrier->name);
            $this->context->smarty->assign('poczta_adr', $address_array);

            if ($carrier->external_module_name == 'owppoczta') {
                return $this->display(OWPPOCZTA_PATH, 'backorder' . OWPPOCZTA_PS_VERSION . '.tpl');
            }
        }
    }

    /**
     * @param $params
     * @return bool|string
     */
    public function hookdisplayBackOfficeTop($params)
    {
        $link = $this->context->link->getModuleLink('owppoczta', 'ajax', array(
            'action' => 'updateDatabase',
            'token' => Configuration::get('MOD_OWPPOCZTA_TOKEN')),
            Configuration::get('PS_SSL_ENABLED') ? true : false);
        $ajax_url = $this->context->link->getModuleLink('owppoczta', 'ajax', array(), Configuration::get('PS_SSL_ENABLED') ? true : false);
        //skrypty w panelu administracyjnym
        $html = '<script type="text/javascript" src="../modules/owppoczta/js/owppoczta_admin.js"></script>';
        $html .= '<script type="text/javascript" >
                var owppoczta_link = "' . $link . '";
                var owppoczta_ajax_url = "' . $ajax_url . '";
        </script>';
        //TODO zmiana ładowania na ajaxowe
        if (Tools::getValue("OWPPOCZTA_NO_UPDATE")) {
            Configuration::set('MOD_OWPPOCZTA_DATABASE_STATUS', 0);
        } elseif (Configuration::get('MOD_OWPPOCZTA_DATABASE_STATUS') > 0 && $this->checkInterval()) {

            return $html . ($this->displayConfirmation('Nie zamykaj okna przeglądarki, w tle odbywa się aktualizacja bazy danych placówek i kodów pocztowych.')) .
            '<style>.module_confirmation.conf.confirm{float:left;}</style><script type="text/javascript">setTimeout( updateOwppocztaDatabase(), 4000);</script>';
        };
        return $html;
    }

    /**
     * @param $params
     */
    public function hookupdateCarrier($params)
    {
        $this->updateConfiguredCarrier((int)$params['id_carrier'], (int)$params['carrier']->id);
    }

    /**
     * @param $old
     * @param $new
     */
    private function updateConfiguredCarrier($old, $new)
    {
        //TODO zaktualizować bazę połączonych gdy zmienia się ID typu przesyłki
    }

    public function hookprocessCarrier($params)
    {
        //TODO aktualizować dane modułu przy zmianach ID przesyłki
    }

    public function checkInterval()
    {
        $last_update_time = Configuration::get('MOD_OWPPOCZTA_LAST_UPDATE');
        $interval = Configuration::get('MOD_OWPPOCZTA_DATABASE_INTERVAL');
        return (time() - $last_update_time) > $interval;
    }

    /**
     * @param null $params
     */
    public function ajaxFrontPostProcess($params = null)
    {
        $action = Tools::getValue("action");
        $action = "ajax_" . $action . "Action";
        if (method_exists($this, $action)) {
            $reflection = new ReflectionMethod($this, $action);
            if ($reflection->isPublic()) {
                echo json_encode($this->$action($params));
            } else {
                echo json_encode("Nie masz prawa korzystać z tej metody!");
            }
        } else {
            echo json_encode("Funkcja nie istnieje!");
        }
        exit();
    }

    /**
     * @return array
     */
    public function ajax_getPostsListAction()
    {
        $postoffice = Tools::getValue("postoffice");
        $page = Tools::getValue("page");
        $page_limit = Tools::getValue("page_limit");
        $id_carrier = (int)($this->context->cart->id_carrier);
        $place_types = OwppocztaConnection::getConnections($id_carrier);
        if (is_numeric(substr($postoffice, 0, 1))) {
            $result_array = OwppocztaPostOffice::get_posts_offices_by_nearest_postcode($postoffice, $place_types, $page_limit, $page);
            $count = OwppocztaPostOffice::get_posts_offices_by_nearest_postcode($postoffice, $place_types, $page_limit, $page, true);
        } else {
            $result_array = OwppocztaPostOffice::get_posts_offices_by_name($postoffice, $place_types, $page_limit, $page);
            $count = OwppocztaPostOffice::get_posts_offices_by_name($postoffice, $place_types, $page_limit, $page, true);
        }
        return array('error' => 0, 'total_count' => $count, 'incomplete_results' => (($page + 1) * $page_limit < $count), 'results' => $result_array);
    }

    /**
     * @param null $pni
     * @return array
     */
    public function ajax_saveChosenPostAction($pni = null)
    {
        //jeśli zapisujemy wybraną pocztę do bazy danych
        //najpierw musimy pobrać aktualny numer koszyka
        $cart_id = (int)($this->context->cookie->id_cart);
        $id_carrier = (int)($this->context->cart->id_carrier);
        //zabezpieczenie przed SQL injection
        if (is_null($pni))
            $pni = (int)Tools::getValue('pni');
        $owppoczta_order = new OwppocztaOrder($cart_id, $pni, $id_carrier);

        return array('success' => $owppoczta_order->save());
    }

    /**
     * @return array
     */
    public function ajax_updateDatabaseAction()
    {
        $is_admin = (is_object(Context::getContext()->controller) && Context::getContext()->controller->controller_type == 'admin');

        $token = Tools::getValue('token');
        if ($is_admin || (!is_null($token) && $token === Configuration::get('MOD_OWPPOCZTA_TOKEN') && Configuration::get('MOD_OWPPOCZTA_DATABASE_STATUS') > 0)) {
            return $this->updateDatabase();
        }
        return array('success' => false);
    }

    /**
     * @return array
     */
    public function ajax_removeLogsAction()
    {
        return array('success' => OwppocztaLogger::removeLogs());
    }

    /**
     * @param bool|false $echo
     * @return array
     */
    public function updateDatabase($echo = false)
    {
        /*
        Zaktualizuj bazę danych z 5 okolicznymi pocztami dla każdego kodu pocztowego - PNAPNIPP
        */
        $return = OwppocztaUpdater::update_database(Configuration::get('MOD_OWPPOCZTA_URI_PNAPNIPP'), DATABASE_PLACOWKI_POCZTY_PNAPNIPP, Configuration::get('MOD_OWPPOCZTA_PP_PNAPNI_ENC'), true);
        /*
        Zaktualizuj bazę placówek pocztowych
        */
        $return = $return && OwppocztaUpdater::update_database(Configuration::get('MOD_OWPPOCZTA_URI_PP'), DATABASE_PLACOWKI_POCZTY, Configuration::get('MOD_OWPPOCZTA_PP_ENC'), true);
        OwppocztaUpdater::cleanTemp();

        /*
        Zapisz czas ostatniej aktualizacji
        */
        if ($return)
            Configuration::updateValue('MOD_OWPPOCZTA_LAST_UPDATE', time());
        /*
        Zapisz logi
        */
        $logs = '';
        foreach (OwppocztaUpdater::getLogs() as $log) {
            $print = OwppocztaLogger::saveLog($log[0], $log[1]);
            if ($echo)
                echo $print;
            $logs .= $print;
        }
        return array('success' => $return, 'message' => $logs);
    }

    /**
     * @return array
     */
    public function ajax_updateConfigurationAction()
    {
        $name = Tools::getValue('name');
        $value = addslashes(Tools::getValue('value'));
        if (!is_null($name) && !is_null($value)) {
            Configuration::updateValue('MOD_OWPPOCZTA_' . $name, $value);
            return array('success' => true,
                'message' => 'Pomyślnie zaktualizowano ustawienia.');
        }
        return array('success' => false);
    }

    /**
     * @return array
     */
    public function ajax_setConnectedAction()
    {
        $id_carrier = Tools::getValue('id_carrier');
        $value = Tools::getValue('value');
        $place_type_name = Tools::getValue('place_type_name');
        if (!is_null($value) && !is_null($id_carrier)) {
            if ($value == 'add' &&
                $this->addConnectedCarrier($id_carrier, $place_type_name) &&
                $this->setCarrierExternalModule($id_carrier)
            )
                return array('success' => true, 'message' => 'Pomyślnie powiązano z metodą przesyłki.');
            elseif ($value == 'remove' &&
                $this->removeConnectedCarrier($id_carrier, $place_type_name) &&
                $this->unsetCarrierExternalModule($id_carrier)
            ) {
                return array('success' => true, 'message' => 'Pomyślnie usunięto powiązanie.');
            }
        }
        return array('success' => false, 'message' => 'Wystąpił błąd!');
    }

    /**
     * @param $cart_id
     * @return bool
     */
    public function setMailTexts($cart_id)
    {
        //@todo sprawdzić czy nie powoduje błędów
        $order = new OwppocztaOrder($cart_id);
        $post_office = OwppocztaPostOffice::get_post_by_pni($order->getPni());

        if (is_array($post_office)) {
            $this->extra_mail_vars = array(
                '{odbior_w_punkcie}' => '<table cellspacing="0" cellpadding="0" style="margin-top: 10px;">
                    <tr>
                        <td style="padding: 10px 0 10px 15px; width: 492px; height:50px; border: 1px solid #930C02; background: no-repeat right bottom url("' . _PS_BASE_URL_ . OWPPOCZTA_URI . 'img/poczta_polska_logo.png") #E71905; padding-left: 12px; color: white;">
                            Wybrana placówka pocztowa:<br/>
                            <strong id="owppoczta-postname">' . $post_office['nazwa_placowki'] . '</strong><br/>
                            ' . $post_office['ulica'] . '<br/>
                            ' . $post_office['kod_pocztowy'] . ' ' . $post_office['miejscowosc'] . '<br/>
                        </td>
                    </tr>
                </table>',
                '{odbior_w_punkcie_tekst}' =>
                    'Wybrana placówka pocztowa: ' . $post_office['nazwa_placowki'] . PHP_EOL . $post_office['ulica'] . PHP_EOL . $post_office['kod_pocztowy'] . ' ' . $post_office['miejscowosc']
            );
            return true;
        } else {
            $this->extra_mail_vars = array(
                '{odbior_w_punkcie}' => '',
                '{odbior_w_punkcie_tekst}' => ''
            );
            return false;
        }
    }

    public function isCarrierConnected($id_carrier)
    {
        return in_array($id_carrier, array_keys($this->getConnectedCarriers()));
    }

    /**
     * Funkcja dodająca owppoczta jako zewnętrzny moduł sposobu przesyłki, aby dodać ją do hooka "extra_carrier"
     *
     * @param $id_carrier
     * @param bool|false $remove
     * @return bool
     */
    protected function setCarrierExternalModule($id_carrier, $remove = false)
    {
        if ($remove)
            $module_name = '';
        else
            $module_name = 'owppoczta';
        return Db::getInstance()->Execute(sprintf('UPDATE ' . _DB_PREFIX_ . 'carrier SET `external_module_name` = "%s" WHERE `id_carrier` = %d', $module_name, $id_carrier));

    }

    /**
     * @param $id_carrier
     * @return bool
     */
    protected function unsetCarrierExternalModule($id_carrier)
    {
        $carrier_types = OwppocztaConnection::getConnections($id_carrier);
        if (empty($carrier_types))
            return $this->setCarrierExternalModule($id_carrier, true);
        return true;
    }
}