<?php

/**
 * Class OwppocztaUpdater
 */
class OwppocztaUpdater
{
    /**
     * @var array
     */
    static private $logs = array();

    /**
     * @param array $log
     */
    static public function addLog(array $log)
    {
        self::$logs[] = $log;
    }

    /**
     * @return array
     */
    static public function getLogs()
    {
        return self::$logs;
    }

    /**
     * @param $source
     * @param $table
     * @param $character_set
     * @param bool $delete
     * @return bool
     */
    static public function update_database($source, $table, $character_set, $delete = false)
    {
        try {
            //uruchom curla
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $source);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $data = curl_exec($ch);
            curl_close($ch);

            //zapisz pobrane archiwum
            $destination = OWPPOCZTA_PATH . 'temp/temp.zip';
            $dirname = dirname($destination);
            if (!is_dir($dirname) || !is_writable($dirname)) {
                self::addLog(array(0, 'Nie udało się zapisać pliku. Sprawdź uprawnienia skryptu dla zapisu w folderze ' . OWPPOCZTA_PATH . 'temp'));
                return false;
            }
            $file = fopen($destination, 'w+');
            fputs($file, $data);
            fclose($file);

            //rozpakuj
            //dodany fallback jeśli na hostingu nie ma biblioteki ziparchive
            $flag = false;
            if (!class_exists('ZipArchive')) {
                require_once(OWPPOCZTA_PATH . 'lib/pclzip.lib.php');
                $zip = new PclZip(OWPPOCZTA_PATH . 'temp/temp.zip');
                $extracted_files = $zip->extract(OWPPOCZTA_PATH . 'temp/'); //bez przypisania powodowało problemy na pewnych wersjach PHP
                $filename = $extracted_files[0]['filename'];
                $flag = true;
            } else {
                $zip = new ZipArchive;
                if ($zip->open(OWPPOCZTA_PATH . 'temp/temp.zip') === TRUE) {
                    $filename = OWPPOCZTA_PATH . 'temp/' . $zip->getNameIndex(0);
                    $zip->extractTo(OWPPOCZTA_PATH . 'temp/');
                    $zip->close();
                    $flag = true;
                }
            }

            if ($flag) {
                if (!$csv = fopen($filename, "r")) {
                    throw new Exception("Could not open $filename for reading.");
                }
                $values = '';

                $first_line = fgets($csv);

                $fields = array_filter(explode(";", rtrim($first_line)));

                /**
                 * Sprawdzenie, czy treść pól jest zamykana cudzysłowiami
                 */
                if (strpos($first_line, "\"") || count($fields) <= 2)
                    $enclosure = "\"";
                else
                    $enclosure = "\t";

                $i = 0;
                while (!feof($csv)) {
                    $line = fgetcsv($csv, 0, ";", $enclosure);
                    if (is_array($line))
                        $values .= iconv($character_set, "UTF-8", '( "' . implode('", "', array_slice($line, 0, count($fields))) . '"),');
                    $i++;
                }

                $sql = 'REPLACE INTO ' . _DB_PREFIX_ . $table . '
                 VALUES ' . rtrim($values, ",") . ';';

            } else {
                self::addLog(array(0, 'Rozpakowanie lub pobranie listy placówek Poczty Polskiej nie powiodło się.'));
                return false;
            }

            if (($delete ? Db::getInstance()->execute('DELETE FROM ' . _DB_PREFIX_ . $table) : true) && Db::getInstance()->execute($sql)) {
                self::addLog(array(10, 'Baza danych ' . _DB_PREFIX_ . $table . ' została zaktualizowana pomyślnie. Zaktualizowano ' . Db::getInstance()->Affected_Rows() . ' linii.'));
                return true;
            } else {//jeśli pojawiły się błędy
                self::addLog(array(0, Db::getInstance()->getMsgError()));
                return false;
            }
        } catch (Exception $e) {
            self::addLog(array(0, $e->getMessage()));
            return false;
        }
    }

    /**
     * Funkcja czyszcząca folder temp z archiwów pobranych przy okazji aktualizacji
     * bazy danych
     */
    static function cleanTemp()
    {
        $files = glob(OWPPOCZTA_PATH . 'temp/*'); // get all file names
        foreach ($files as $file) { // iterate files
            if (is_file($file))
                unlink($file); // delete file
        }
    }
}